// **************************************************************************************
// el Comandante                                                    guerilla.bplaced.net
// --------------------------------------------------------------------------------------
// Information:
// Simple infrared detector, without any signal decoding. This circuit and the software
// is an experiment and not suitable for outdoor / long range / accurate measurements.
// Anyway, the precision under stable conditions is a suprise.
// The circuit detects IR-Remote signals, cigarette embers...
// The drawback is the influence of ambient light. Best results indoor at low light.
// Keep in mind that the spectrum of normal daylight disturbs the infrared reception.
//
// Look at the circuit schematic:
// The Phototransistor is an old SFH 303FA-3/4 type. Using another type, most likely,
// you must adjust the PRECISION POTENTIOMETER R2 and/or the variable IRThreshold.
// Set the variable AmbientLightCheck to 1 to feedback the ambient light (Sensor value)
// and enable the calculation of a new offset, marked with a vertical line.
// Adjust the Sensor value near 0 with R2, to adapt to the ambient light.
// To improve the circuit and automate the adjustment replace R2 with a LDR, i. e.:
// Type: A 1060 13 or A 9060 13 - Specs: R at 10 Lux = 27 KΩ - 94 KΩ - More sensitive
// Type: A 1050 12 or A 9050 12 - Specs: R at 10 Lux = 18 KΩ - 44 KΩ - Less sensitive
// Type: FW300                  - Specs: R at 10 Lux =  8 KΩ - not checked
// The LDR should not placed beside the phototransistor!
//
// Play with the variable IRThreshold.
// Using a resistor as R2, IRThreshold ~ 100 was a good value, while testing indoor.
// Using a LDR as R2, IRThreshold ~ 50 was a good value, while testing indoor.
//
// The LED connected to GPIO4 is switched on if: Sensor >= Offset + Threshold
// Don't use a LED with an electrical current above 30mA (esp32 max. = 40mA).
// LED resistor R1 calculation example (read the LED datasheet):
// Supply Voltage = 3.3 Volt
// LED Voltage = 1.2 Volt
// LED Current = 10mA (0.01A)
// R1 = (3.3 - 1.2) / 0.01
// R1 = 210 Ohm
// The next higher value in the E24 resistor table is 220 Ohm.
// --------------------------------------------------------------------------------------
// Used Libraries:
// TFT_eSPI                 --> https://github.com/Bodmer/TFT_eSPI
// **************************************************************************************

#include <TFT_eSPI.h>                           // TFT_espi library
TFT_eSPI tft = TFT_eSPI();
TFT_eSprite graph = TFT_eSprite(&tft);          // Plotter

// ######################################################################################
// Change values in this area to match your envoirement and desires
// --------------------------------------------------------------------------------------
// Message delay, time to test the LED at startup, time to display offset calculation...
uint16_t MessageDelay = 3000;
// Perform an ambient light check to (auto) adjust the offset, 0 = disable, 1 = enable
uint8_t AmbientLightCheck = 1;
// Plotting cycles after new a offset calculation performed, good values are 50 - 100
uint8_t CounterMax = 100; // max. 255
// Delay to read new sensor value, senseless because plotting graphics takes time
uint32_t IRSensorDelay = 100;
// Threshold to switch LED on
uint16_t IRThreshold = 100;
// Analog input 12 Bit (default), maximum value, float for calculations in graphDraw()
float const irSensorMax = 4095.0; //2047.0;//1023.0; = zoom graphic
// Define the anolog input pin and the digital outpin pin, must match your hardware
#define IRSensor_PIN 34 // ESP32 analog input pin GPIO34 (ADC1_CH6)
#define LED_PIN 4       // ESP32 digital output pin GPIO4
// ######################################################################################

// Vars
uint32_t  update;
uint32_t  i;
uint32_t  x;
uint32_t  irOffset;
uint16_t  irOffsetArray[0x100];
uint16_t  irSensorValue;                                      // analog input
uint16_t  tftWidth;
uint16_t  tftHeight;
uint16_t  graphWidth;
uint16_t  graphHeight;
uint16_t  graphOldY;
uint8_t   Counter;
uint8_t   AmbientLightDrawLine;
uint32_t  AmbientLightAverageValue;


// **************************************************************************************
// Setup
void setup(void)
{
  pinMode(TFT_BL, OUTPUT);                                // GPIO - init pin
  pinMode(LED_PIN, OUTPUT);                               // GPIO - init pin
  //pinMode(IRSensor_PIN, INPUT_PULLUP);                    // ?
  //irSensorValue = analogRead(IRSensor_PIN);               // ?
  digitalWrite(TFT_BL, 0);                                // Switch backlight off
  tft.init();                                             // TFT_espi - init
  screenInit();                                           // check tft resolution
  screenBuild();                                          // build main screen
  graphCreate();                                          // create graphic
  digitalWrite(TFT_BL, 1);                                // switch backlight on
  LEDTest();                                              // test LED
  CalcOffset();                                           // calculate IR offset
  update = millis() + IRSensorDelay;
}


// **************************************************************************************
// Main loop
void loop()
{
  if (update <= millis())
  {
    irSensorValue = analogRead(IRSensor_PIN);             // read anolog input
    tft.setCursor(tftWidth / 20, tftHeight - tftHeight / 100.0 * 10);
    tft.printf("%d   ", irSensorValue);                   // print value (0 - 4095)
    AmbientLight();                                       // perform ambient light check
    graphDraw();                                          // draw graphic
    if (irSensorValue >= (irOffset + IRThreshold))        // if offset + threshold reached
    {digitalWrite(LED_PIN, 1);}                           // LED on
    else
    {digitalWrite(LED_PIN, 0);}                           // LED off
    update = millis() + IRSensorDelay;
  }
}


// **************************************************************************************
// Screen - size
void screenInit()
{
  tft.setRotation(3);                                     // set desired rotation
  tft.fillScreen(0x0000);                                 // set desired background color
  tftWidth = tft.width();                                 // get tft width
  tftHeight = tft.height();                               // get tft height
  tft.setTextFont(1);
  if (tftWidth >= 320 || tftHeight >= 320)                // big screen (i.e. WT32-SC01 - 480 x 320)
  {tft.setTextSize(2);}
  else if (tftWidth < 320 || tftHeight < 320)             // small screen (i.e. LILYGO TTGO-T-Display - 240 x 135)
  {tft.setTextSize(1);}
}


// **************************************************************************************
// Screen - build
void screenBuild()
{
  tft.setTextColor(0xFFFE, 0x0000);
  tft.setTextDatum(TL_DATUM);
  tft.drawLine(0, tftHeight / 100.0 * 13, tftWidth, tftHeight / 100.0 * 13, 0x8410);
  tft.drawLine(0, tftHeight - tftHeight / 100.0 * 24, tftWidth, tftHeight - tftHeight / 100.0 * 24, 0x8410);
  tft.setCursor(tftWidth / 20, tftHeight - tftHeight / 100.0 * 19);
  tft.print("Sensor");
  tft.setCursor(tftWidth / 20.0 * 5, tftHeight - tftHeight / 100.0 * 19);
  tft.print("Offset +");
  tft.setCursor(tftWidth / 20.0 * 9.5, tftHeight - tftHeight / 100.0 * 19);
  tft.print("Threshold ->");
  tft.setCursor(tftWidth / 20.0 * 16, tftHeight - tftHeight / 100.0 * 19);
  tft.print("LED On");
}


// **************************************************************************************
// LED - test LED functionality
void LEDTest()
{
  tft.setTextColor(0xFFFE, 0x0000);
  tft.drawString("Testing LED...", 4, tftHeight / 100.0 * 6);
  digitalWrite(LED_PIN, 1);                               // LED on
  delay(MessageDelay);                                    // test LED for the specified time
  digitalWrite(LED_PIN, 0);                               // LED off
}


// **************************************************************************************
// IR - calculating sensitivity offset, the envoirement light
void CalcOffset()
{
  tft.fillRect(0, 0, tftWidth, tftHeight / 100.0 * 12, 0x0000);
  tft.setTextColor(0xFFFE, 0x0000);
  tft.drawString("Offset calculation, keep calm...", 4, tftHeight / 100.0 * 6);

  tft.setTextColor(0x0714, 0x0000);
  irOffset = 0;
  for (i =0; i < CounterMax; i++)                         // read anolog input to array
  {
    irOffsetArray[i] = analogRead(IRSensor_PIN);
    irOffset = irOffset + irOffsetArray[i];
    tft.setCursor(tftWidth / 20.0 * 5, tftHeight - tftHeight / 100.0 * 10);
    tft.printf("%d   ", irOffsetArray[i]);
    delay(50);
  }
  irOffset = (irOffset + 1) / CounterMax;                 // calculate average offset value

  tft.setTextColor(0xFFFE, 0x0000);
  tft.setCursor(tftWidth / 20.0 * 5, tftHeight - tftHeight / 100.0 * 10);
  tft.printf("%d   ", irOffset);
  tft.setCursor(tftWidth / 20.0 * 9.5, tftHeight - tftHeight / 100.0 * 10);
  tft.print(IRThreshold);
  tft.setCursor(tftWidth / 20.0 * 16, tftHeight - tftHeight / 100.0 * 10);
  tft.print(irOffset + IRThreshold);

  delay(MessageDelay);

  tft.fillRect(0, 0, tftWidth, tftHeight / 100.0 * 12, 0x0000);
  tft.drawString("Ready to receive IR signals", 4, tftHeight / 100.0 * 6);
}


// **************************************************************************************
// Ambient light - auto adjust
void AmbientLight()
{
  if (AmbientLightCheck == 1)
  {
    x = 0;
    if (irSensorValue < irOffset)                         // if sensor value below offset
    {x = irOffset - irSensorValue;}                       // difference

    if (irSensorValue > irOffset || x > 50)               // ?
    {
      if (Counter < CounterMax)                           // read analog input to array
      {
        irOffsetArray[Counter] = irSensorValue;
        Counter++;
      }
      else
      {
        AmbientLightAverageValue = 1;                     // prevent division by 0
        for (i = 0; i < CounterMax; i++)                  // calculate average offset value
        {AmbientLightAverageValue = AmbientLightAverageValue + irOffsetArray[i];}
        irOffset = AmbientLightAverageValue / CounterMax; // set new offset

        tft.setCursor(tftWidth / 20.0 * 5, tftHeight - tftHeight / 100.0 * 10);
        tft.printf("%d   ", irOffset);                    // update screen -> Offset
        tft.setCursor(tftWidth / 20.0 * 16, tftHeight - tftHeight / 100.0 * 10);
        tft.printf("%d   ", irOffset + IRThreshold);      // update screen -> LED on
        Counter = 0;                                      // reset counter
        AmbientLightDrawLine = 1;                         // set flag for drawing vertical line
      }
    }

    else                                                  // sensor value not all time below or above current offset
    {Counter = 0;}                                        // prevents short brighter or darker ambient light events
  }
}


// **************************************************************************************
// Graphic - draw
void graphDraw()
{
    graph.setScrollRect(0, 0, graphWidth, graphHeight, 0x0100);
    graph.scroll(-1, 0);

    if (irSensorValue <= irSensorMax)
    {x = (graphHeight - 1) - ((graphHeight - 1) / irSensorMax) * irSensorValue;}
    else
    {x = 0;}

    graph.drawLine(graphWidth-2, graphOldY, graphWidth-1, x, 0xF800);
    graphOldY = x;

    if (AmbientLightDrawLine == 1)                        // offset changed
    {
      graph.drawLine(graphWidth-1, 0, graphWidth-1, graphHeight - 1, 0x8410);
      AmbientLightDrawLine = 0;
    }

    graph.pushSprite(0, tftHeight / 100.0 * 18);
}


// **************************************************************************************
// Graphic - create
void graphCreate()
{
  graphWidth = tftWidth;
  graphHeight = tftHeight / 100.0 * 54;
  graph.setColorDepth(8);                                 // 8 bit colors minimize required RAM
  graph.createSprite(graphWidth, graphHeight);
  graph.fillSprite(0x0100);
}
